@@ -75,26 +75,30 @@ check.collinearity <- function (proposal, features, F.0.size, data, mock) {
   # Add the proposal to the feature list for evaluation
   features[[length(features) + 1]] <- proposal
   # Generate mock data to test with (avoiding too costly computations)
-  if (mock)
-    mock.data <- matrix(c(runif((F.0.size * 2), -100, 100), rep(1, F.0.size * 2),
-                        runif((F.0.size * 2) * (F.0.size), -100, 100)), F.0.size * 2, F.0.size + 2)
-  else
-    mock.data <- check.data(data[seq_len(min(F.0.size * 2, dim(data)[1])), ], FALSE)
+  n <- F.0.size * 2
+  if (mock) {
+    mock.data <- list(x = matrix(runif(n * (F.0.size + data$fixed), -100, 100), n, F.0.size + data$fixed),
+                      y = matrix(runif(n * (ncol(data$y)), -100, 100), n, ncol(data$y)),
+                      fixed = data$fixed)
+  } else {
+    obs_idx <- seq_len(min(n, nrow(data$x)))
+    mock.data <- list(x = data$x[obs_idx, ], y = data$y[obs_idx, ], fixed = data$fixed)
+  }
   # Use the mock data to precalc the features
   mock.data.precalc <- precalc.features(mock.data, features)
   # Fit a linear model with the mock data precalculated features
-  linearmod <- lm(as.data.frame(mock.data.precalc[, -2]))
+  linearmod <- lm.fit(mock.data.precalc$x, mock.data.precalc$y)
   # Check if all coefficients were possible to calculate
   if (sum(is.na(linearmod$coefficients)) == 0) return(FALSE)
   else return(TRUE)
 }
 
 # Generate features to represent the covariates, just takes the count needed
-gen.covariates <- function (count) {
+gen.covariates <- function (data) {
   features <- list()
-  for (i in 1:count) {
+  for (i in seq_len(ncol(data$x) - data$fixed)) {
     features <- c(features, i)
-    class(features[[i]]) <- "feature"
+    class(features[[length(features)]]) <- "feature"
   }
   return(features)
 }
\ No newline at end of file
