@@ -13,10 +13,9 @@ simulated.annealing <- function (model, data, loglik.pi, indices, complex, param
   temp <- params$t.init # Initial temperature
 
   # Calculate current likelihood
-  model.res <- loglik.pre(loglik.pi, model, complex, data, loglikparams,visited.models, sub)
+  model.res <- loglik.pre(loglik.pi, model, complex, data, loglikparams, visited.models, sub)
   model.lik <- model.res$crit
-  models[[length(models) + 1]] <- list(prob=NA, model=model, coefs=model.res$coefs, crit=model.lik, alpha=NA)
-  # print(paste("SA Start:", model.lik))
+  models[[length(models) + 1]] <- list(prob = NA, model = model, coefs = model.res$coefs, crit = model.lik, alpha = NA)
   while (temp > params$t.min) {
     # Make M tries at current temperature
     for (m in 1:params$M) {
@@ -25,7 +24,7 @@ simulated.annealing <- function (model, data, loglik.pi, indices, complex, param
       model.proposal <- loglik.pre(loglik.pi, proposal, complex, data, loglikparams, visited.models = visited.models, sub = sub)
       proposal.lik <- model.proposal$crit
       # Store the model that we have calculated
-      models[[length(models) + 1]] <- list(prob=NA, model=proposal, coefs=model.proposal$coefs, crit=proposal.lik, alpha=NA)
+      models[[length(models) + 1]] <- list(prob = NA, model = proposal, coefs = model.proposal$coefs, crit = proposal.lik, alpha = NA)
       # Calculate move probability for negative steps (Bolzmann distribution, see Blum and Roli p. 274)
       if (proposal.lik > model.lik) alpha <- 1
       else alpha <- min(1, exp((proposal.lik - model.lik) / temp))
@@ -38,8 +37,7 @@ simulated.annealing <- function (model, data, loglik.pi, indices, complex, param
     # Update temperature
     temp <- temp * exp(-params$dt)
   }
-  # print(paste("SA Finish:", model.lik))
-  return(list(model=model, kern=kernel, models=models))
+  return(list(model = model, kern = kernel, models = models))
 }
 
 greedy.optim <- function (model, data, loglik.pi, indices, complex, params, loglikparams, kernel = NULL, visited.models = NULL, sub = FALSE) {
@@ -82,10 +80,10 @@ greedy.optim <- function (model, data, loglik.pi, indices, complex, params, logl
 
 local.optim <- function (model, data, loglik.pi, indices, complex, type, params, kernel = NULL, visited.models = NULL, sub = FALSE) {
   if (type == 1) {
-    return(simulated.annealing(model, data, loglik.pi, indices, complex, params$sa, params$loglik, kernel, visited.models = visited.models, sub = sub))
+    return(simulated.annealing(model, data, loglik.pi, indices, complex, params$sa, params$mlpost, kernel, visited.models = visited.models, sub = sub))
   }
   if (type == 2) {
-    return(greedy.optim(model, data, loglik.pi, indices, complex, params$greedy, params$loglik, kernel, visited.models = visited.models, sub = sub))
+    return(greedy.optim(model, data, loglik.pi, indices, complex, params$greedy, params$mlpost, kernel, visited.models = visited.models, sub = sub))
   }
   if (type == 3) {
     return("not implemented")
